
// a simple class for string trees.
// This version keeps the size of each subtree in each node's count_ attribute.
// count_ is to support an O(log N) implementation of Get(k)

class TreeDict {  // StrTree with count attribute
     
    // We do not provide a way to set the string in an existing node; 
    // Providing such a method is a bad idea for ordered structures (why?)
    class TreeNode {
    private String key_;
    private int count_;     // should always be the size of this SUBTREE
    private TreeNode left_, right_;
    public TreeNode(String d, int c, TreeNode l, TreeNode r) {
        key_ = d; 
        count_= c;
        left_ = l; 
        right_= r;
    }
    public String key() {return key_;}
    public int   value() {return count_;}
    public TreeNode left()  {return left_;}
    public TreeNode right() {return right_;}
    public void setvalue(int count) {count_ = count;}
    public void setleft  (TreeNode t) {left_ = t;} // doesn't update count_!
    public void setright (TreeNode t) {right_ = t;} // doesn't update count_!
    }

    private TreeNode root_ = null;

    public void inTraverse() {inTraverse(root_, 0);}

    private void inTraverse(TreeNode p, int depth) {
        int indent = 4;
        if (p==null) return;
        inTraverse(p.left(), depth+1);
        String prefix = "";
        for (int i=0; i<depth*indent; i++) prefix += ' ';
        System.out.printf("%s%s  (%d)%n", prefix, p.key(), p.value());
        inTraverse(p.right(), depth+1);
    }

    // must not be called with val == null
    public void insert(String val, int c) {
        if (root_ == null) {
            root_ = new TreeNode(val, c, null, null); // sets count_, but not multiplicity_
            return;
        } else {
            rinsert(val, c, root_);
        }
    }

  // recursive insert: does not get called with t==null
  // returns true if the value was actually inserted (that is, it was NOT already there)
    private boolean rinsert(String k, int c, TreeNode t) {
        int comp = k.compareTo(t.key()); // = 0 if Strings ==, <0 if k<t.key, etc
        if (comp == 0 ) {
            return false;       // already there
        } else if (comp < 0) {      // k < t.key()
            if (t.left() == null) {
                t.setleft(new TreeNode(k, c, null, null));  // t.left()'s count is now 1
                return true;
            } else {
                return rinsert(k, c, t.left());
            }
        } else { // val > t.key(); comp > 0
            if (t.right() == null) {        
                t.setright(new TreeNode(k, c, null, null));
                return true;
            } else {                
                return rinsert(k, c, t.right());
            }
        }
    }

    public boolean containsKey(String k) {
        return containsKey(k, root_);
    }
    
    private boolean containsKey(String k, TreeNode p) {
        if (p == null) return false;
        int comp = k.compareTo(p.key());
        if (comp == 0) {    // found it
            return true;
        } else if (comp < 0) {  // k < p.key()
            return containsKey(k, p.left());
        } else {
            // finish this part
            return false;   // here only to make this stub compile
        }
    }
    
    public int get(String k) {return get(k, root_);}
    
    private int get(String k, TreeNode p) {
        if (p == null) return -1;
        int comp = k.compareTo(p.key());
        if (comp == 0) {    // found it
            return p.value();
        } else if (comp < 0) {  // finish from here
            
        }
        return 0;  // here only to make this stub compile
    }
    
    public void put(String k, int v) {
        if (root_ == null) {
            root_ = new TreeNode(k, v, null, null);
        } else {
            put(k, v, root_);
        }
    }

    private void put (String k, int v, TreeNode p) {
        int comp = k.compareTo(p.key());
        // finish; use rinsert() as a model
    }

}



